"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const request = require("superagent");
const _ = require("lodash");
const klg_logger_1 = require("klg-logger");
const klg_retry_1 = require("klg-retry");
const logger = new klg_logger_1.Logger({
    level: 'info',
    dateformat: 'yyyy-mm-dd HH:MM:ss.L'
});
const HTTP_METHOD = {
    POST: 'post',
    GET: 'get',
    PUT: 'put'
};
class Request {
    constructor(config) {
        // 默认配置
        this.config = {
            retries: 4,
            factor: 2,
            minTimeout: 1000,
            timeOut: 60000,
            retryWhen50x: true,
            retryWhenTimeout: false,
            retryWhenConnectError: true
        };
        Object.assign(this.config, config);
        this.retry = new klg_retry_1.Retry();
    }
    async sendData(url, data) {
        if (this.config.beforeSend) {
            data = await this.config.beforeSend(data);
        }
        let { interfaceName, server, body, query, options } = data;
        if (!interfaceName) {
            interfaceName = _.last(url.split('/'));
            data.interfaceName = interfaceName;
            data.url = url;
        }
        let response = null;
        let httpMethod = options.httpMethod;
        let startTime = Date.now();
        try {
            logger.info(`request[${httpMethod}] to ${url}`);
            let operate;
            if (httpMethod === HTTP_METHOD.POST || httpMethod === HTTP_METHOD.PUT) {
                operate = request[httpMethod](url)
                    .type(options.postType || '')
                    .send(body);
            }
            if (httpMethod === HTTP_METHOD.GET) {
                operate = request.get(url).query(query);
            }
            const res = await operate
                .timeout(this.config.timeOut)
                .set('Accept', options.accept)
                .set(options.headers || {})
                .then();
            response = JSON.parse(res.text);
        }
        catch (err) {
            logger.info('request err', url || interfaceName || server || 'none', err.message);
            logger.info('request err', err.stack);
            response = { err: err.message, status: err.status };
        }
        if (this.config.afterSend) {
            data.useTime = Date.now() - startTime;
            data.response = response;
            response = await this.config.afterSend(data);
        }
        return response;
    }
    async sendDataRetry(url, data) {
        let res = null;
        try {
            await this.retry.using(async () => {
                res = await this.sendData(url, data);
                if (this.shouldRetry(res)) {
                    if (!data.retryTimes)
                        data.retryTimes = 0;
                    data.retryTimes += 1;
                    throw new Error('need retry');
                }
            }, this.config);
        }
        catch (e) {
            logger.info('重试超过最大次数 exit');
        }
        finally {
            return res;
        }
    }
    shouldRetry(res) {
        if (!res || !res.err)
            return false;
        const config = this.config;
        return Boolean(shouldRetryConnectError(res.err) || shouldRetryStatus50x(res) || shouldRetryTimeOut(res.err));
        function shouldRetryConnectError(err) {
            return config.retryWhenConnectError && (~err.indexOf('getaddrinfo ENOTFOUND') ||
                ~err.indexOf('connect ECONNREFUSED') ||
                ~err.indexOf('connect ETIMEDOUT') ||
                ~err.indexOf('read ECONNRESET'));
        }
        function shouldRetryStatus50x(res) {
            return config.retryWhen50x && res.status && (res.status.toString().match(/5\d{2}/));
        }
        function shouldRetryTimeOut(err) {
            return config.retryWhenTimeout && (~err.indexOf('timeout') ||
                ~err.indexOf('Timeout'));
        }
    }
    async get(url, data) {
        data.options = data.options || {};
        Object.assign(data.options, {
            accept: 'application/json',
            httpMethod: HTTP_METHOD.GET
        });
        return await this.sendDataRetry(url, data);
    }
    async post(url, data) {
        data.options = data.options || {};
        Object.assign(data.options, {
            accept: 'application/json',
            postType: 'json',
            httpMethod: HTTP_METHOD.POST
        });
        return await this.sendDataRetry(url, data);
    }
    async put(url, data) {
        data.options = data.options || {};
        Object.assign(data.options, {
            accept: 'application/json',
            postType: 'json',
            httpMethod: HTTP_METHOD.PUT
        });
        return await this.sendDataRetry(url, data);
    }
}
exports.Request = Request;
//# sourceMappingURL=Request.js.map